import sys, os

from eclaircie import *

dest_filename = os.path.join(os.path.dirname(__file__), "generated_page.rst")

if need_update([__file__], [dest_filename]):
  write_file(dest_filename, """
.. lang:: fr

Page générée
%%%%%%%%%%%%

Cette page est générée à partir d'un script Python :

.. lang:: en

Generated page
%%%%%%%%%%%%%%

This page is generated by a Python script:

.. lang:: all

.. code-block:: python

   import sys, os

   from eclaircie import *

   dest_filename = os.path.join(os.path.dirname(__file__), "generated_page.rst")

   if need_update([__file__], [dest_filename]):
       write_file(dest_filename, " < put your text here > ")

""")



# éClaircie
# Copyright (C) 2014-2018 Jean-Baptiste LAMY

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os, locale
import PIL, PIL.Image
from datetime import date as Date
from collections import defaultdict

from eclaircie.email_obfuscator import *
import eclaircie.rst_ext

VERSION = "0.3"
EC_THEME_PATH = os.path.join(os.path.dirname(__file__), "themes")

def need_update(origs, deriveds):
  for i in range(len(deriveds)):
    if not os.path.lexists(deriveds[i]):
      deriveds[i] = "%s.empty" % os.path.splitext(deriveds[i])[0]
      if not os.path.lexists(deriveds[i]): return True
      
  orig_mtime    = max((os.path.getmtime(orig)    for orig    in origs if os.path.exists(orig)), default = float("inf"))
  derived_mtime = min((os.path.getmtime(derived) for derived in deriveds))
  return orig_mtime > derived_mtime

def write_file(filename, s, touch = False, touch_ref = ""):
  if isinstance(s, str): s = s.encode("utf8")
  if os.path.lexists(filename) and (len(s) == os.path.getsize(filename)):
    old_s = open(filename, "rb").read()
    if s == old_s:
      if touch:
        if os.path.getmtime(filename) < os.path.getmtime(touch_ref):
          print("touch    %s" % filename)
          os.utime(filename)
      return False
    
  print("write    %s" % filename)
  os.makedirs(os.path.dirname(filename), exist_ok = True)
  open(filename, "wb").write(s)
  return True

def reduce_image(filename, dest_filename, max_width = 200, max_height = 150, copy_if_already_small_enough = False):
  if need_update([filename], [dest_filename]):
    if not filename.endswith(".svg"):
      image = PIL.Image.open(filename)
      if (image.size[0] > max_width) or (image.size[1] > max_height):
        if image.size[0] * max_height  > image.size[1] * max_width:
          if int(image.size[0] * max_height / image.size[1]) > max_width * 1.4:
            image.thumbnail((max_width, int(image.size[1] * max_width / image.size[0])), 1)
          else:
            image.thumbnail((100000, max_height), 1)
        else:
          image.thumbnail((100000, max_height), 1)
        print("create   %s" % dest_filename)
        image.save(dest_filename)
        return
        
    if copy_if_already_small_enough:
      print("create   %s" % dest_filename)
      data = open(filename, "rb").read()
      open(dest_filename, "wb").write(data)
      


class Doc(object):
  def __init__(self, blog, src_filename):
    self.blog           = blog
    self.src_filename   = src_filename
    self.categories     = []
    self.doc_name       = os.path.splitext(os.path.relpath(src_filename, blog.blog_dir))[0]
    self.empty_filename = "%s.empty" % os.path.join(blog.blog_dir, self.doc_name)
    self.empty          = os.path.exists(self.empty_filename) and (os.path.getsize(self.empty_filename) == 0)
    self.html_pages     = None
    self.changed        = False
    self.need_update    = False
    self.html_filenames = {}
    self.extra_html_pages = {}
    
    for path, theme in self.blog.ec_multiple_themes:
      if self.doc_name.startswith(path):
        self.theme = theme
        break
    
    if len(blog.langs) == 1:
      f = src_filename[len(blog.blog_dir):]
      if f.startswith(os.sep): f = f[1:]
      f = os.path.join(blog.blog_dir, "html", f)
      f = "%s.html" % os.path.splitext(f)[0]
      self.html_filenames[blog.langs[0]] = f
    else:
      for lang in blog.langs:
        f = src_filename[len(blog.blog_dir):]
        if f.startswith(os.sep): f = f[1:]
        f = os.path.join(blog.blog_dir, "html", f)
        f = "%s_%s.html" % (os.path.splitext(f)[0], lang.langs[0])
        self.html_filenames[lang] = f
        
    blog.docs.append(self)
    
  def __repr__(self):
    extra = ""
    if self.theme and (not self.theme is self.blog.ec_multiple_themes[-1][1]): extra += " theme=%s" % self.theme.name
    #extra += " theme=%s" % self.theme.name
    if self.need_update: extra += " NEED UPDATE"
    return "<%s '%s'%s>" % (self.__class__.__name__, self.src_filename, extra)
    
  def add_link(self, link_filename): raise ValueError("Symlink are supported only for posts.")
  
  def ready(self):
    if "/" in self.doc_name: self.categories = [self.blog.categories_dict[os.path.dirname(self.doc_name)]]
    else:                    self.categories = [self.blog.categories_dict[""]]
    
  def get_src_filenames(self): return [self.src_filename]

  def assert_need_update(self): self.need_update = True
  
  def check_update(self, force = False):
    if force:
      print("changed  %s (force)" % self.src_filename)
      self.changed = self.need_update = True
    else:
      html_filenames = [filename for filename in self.html_filenames.values() if os.path.exists(filename)]
      if (not html_filenames) or need_update(self.get_src_filenames(), html_filenames):
        self.changed = self.need_update = True
        print("changed  %s" % self.src_filename)
        
  #def update_sibling(self):
  #  for category in self.category:
  #    for doc in category.post, category.pages:
  #      doc.need_update = True
        
  def get_html_page(self, lang):
    html_pages = self.get_html_pages()
    for sublang in lang.langs:
      html_page = html_pages.get(LANGS[sublang])
      if html_page: return html_page
    return None
  
  def get_html_pages(self):
    if self.html_pages is None:
      self.html_pages = {}
      src = self.get_source()
      lang_2_rst = split_lang(src)
      
      docs = []
      doc = self
      while doc:
        docs.insert(0, doc)
        doc = doc.categories[0]
        
      for lang, rst in lang_2_rst.items():
        html_page = self.create_html_page(docs, lang, rst)
        self.html_pages[lang] = html_page
        
    return self.html_pages
  
  def create_html_page(self, docs, lang, rst):
    return HTMLPage(self.html_filenames[lang], docs, lang, rst, self.theme)
  
  def build(self):
    if self.need_update:
      print("build    %s as %s" % (self.src_filename, self.__class__.__name__))
      self.get_html_pages() # Generate HTML pages
      
  def get_source(self):
    print("read     %s" % self.src_filename)
    return open(self.src_filename).read()
  
  def save_html_pages(self):
    if self.need_update and self.html_pages:
      for html_page in self.html_pages.values():
        write_file(html_page.filename, html_page.get_html_page(), touch = True, touch_ref = self.src_filename)
      for html_pages in self.extra_html_pages.values():
        for html_page in html_pages:
          write_file(html_page.filename, html_page.get_html_page())
          
class Page(Doc):
  def ready(self):
    super().ready()
    for category in self.categories: category.pages.append(self)
    
class Archives(Page):
  def check_update(self, force = False):
    super().check_update(force)
    for category in self.categories:
      for doc in category.posts_rec: # Archives are placed at the end => ok
        if doc.changed: self.changed = self.need_update = True
        break
      
class Include(Doc):
  def ready(self):
    super().ready()
    for category in self.categories: category.includes.append(self)

class CommentsInclude(Include):
  def ready(self):
    self.post = self.blog.src_filename_2_doc.get(self.src_filename[:-13] + ".rst")
    if self.post:
      self.post.comments_include = self
    else:
      print("WARNING: comments include for unknown post: '%s'!" % self.src_filename)
      
  def assert_need_update(self): pass
  def check_update(self, force = False): pass
  
  def build(self): pass
  
  def get_html_pages(self): return {}
  
class Search(Page):
  def ready(self):
    super().ready()
    for category in self.categories: category.pages.append(self)
    self.blog.search_doc = self
    
class Post(Doc):
  def __init__(self, blog, src_filename, date):
    super().__init__(blog, src_filename)
    self.date = date
    self.link_filenames = []
    self.comments_include = None
    
  def formated_date(self): return self.date.strftime("%x")
  
  def add_link(self, link_filename):
    self.link_filenames.append(link_filename)
    
  def get_src_filenames(self): return [self.src_filename, "%s_comments.inc" % self.src_filename[:-4]]
    
  def ready(self):
    super().ready()
    
    for link_filename in self.link_filenames:
      link_doc_name = os.path.splitext(os.path.relpath(link_filename, self.blog.blog_dir))[0]
      if "/" in link_doc_name: self.categories.append(self.blog.categories_dict[os.path.dirname(link_doc_name)])
      else:                    self.categories.append(self.blog.categories_dict[""])
      
    ancestor_categories = set()
    for category in self.categories:
      category.posts.append(self)
      while category:
        ancestor_categories.add(category)
        if category.doc_name in self.blog.ec_dont_propagate_posts_for_categories: break
        category = category.categories[0]
        
    for ancestor_category in ancestor_categories: ancestor_category.posts_rec.append(self)
    
  def check_update(self, force = False):
    super().check_update(force)
    if self.comments_include:
      self.comments_include.check_update(force)
      if self.comments_include.need_update: self.need_update = True
    
  def create_html_page(self, docs, lang, rst):
    if self.comments_include:
      comments = self.comments_include.get_source()
      lang_2_comments = split_lang(comments)
      comments = lang_2_comments.get(lang, "")
    else:
      comments = ""
      
    return HTMLPage(self.html_filenames[lang], docs, lang, rst, self.theme, comments = comments)
      
  
class Category(Doc):
  def __init__(self, blog, src_filename):
    super().__init__(blog, src_filename)
    self.subcategories = []
    self.pages         = []
    self.includes      = []
    self.posts         = []
    self.posts_rec     = [] # Posts, including posts in subcategories
    if "/" in self.doc_name: self.category_name = self.doc_name.rsplit("/", 1)[0] # Remove "/index"
    else:                    self.category_name = "" # Root category
    if   "/" in self.category_name: self.categories = [blog.categories_dict[self.category_name.rsplit("/", 1)[0]]]
    elif self.category_name:        self.categories = [blog.categories_dict[""]]
    else:                           self.categories = [None]  # Root category
    if self.categories[0]: self.categories[0].subcategories.append(self)
    
    blog.categories_dict[self.category_name] = self
    
  def ready(self):
    pass # Disable Doc's behavior
    self.subcategories.sort(key = lambda category: category.doc_name)
    self.pages        .sort(key = lambda page:     page.doc_name)
    self.posts        .sort(key = lambda post:     post.date)
    self.posts_rec    .sort(key = lambda post:     post.date)
    
  def check_update(self, force = False):
    super().check_update(force)
    
    cascade_update = self.changed
    
    archives = []
    for doc in self.pages + self.includes + self.posts + self.subcategories:
      if isinstance(doc, Archives): archives.append(doc)
      else:
        doc.check_update(force)
        #if doc.need_update:
        if doc.changed:
          self.need_update = True
          if isinstance(doc, Page) or isinstance(doc, Category):
            cascade_update = True # Page/category title might have changed
            
    for doc in archives: # Archives page need to be checked AFTER the index, not before, because it depends on its posts.
      doc.check_update(force)
      if doc.need_update:
        self.need_update = True
        cascade_update = True
        
    if not self.need_update:
      for post in self.posts_rec:
        if post.changed:
          self.need_update = True
          break
        
    if cascade_update:
      for doc in self.pages + self.includes + self.posts + self.subcategories:
        doc.assert_need_update()
        
  def assert_need_update(self):
    super().assert_need_update()
    for doc in self.pages + self.includes + self.posts + self.subcategories: doc.assert_need_update()
    
  def get_posts_rec(self, lang):
    r = []
    for post in self.posts_rec:
      if post.get_html_page(lang): r.append(post)
    r.sort(key = lambda doc: doc.date)
    r.reverse()
    return r
  
  def create_html_page(self, docs, lang, rst):
    posts_rec = self.get_posts_rec(lang)
    if len(posts_rec) <= self.blog.number_of_recent_post:
      recents = posts_rec
    else:
      recentss = [[]]
      for post in posts_rec:
        if post.empty: continue
        recentss[-1].append(post)
        if len(recentss[-1]) >= self.blog.number_of_recent_post: recentss.append([])
      if not recentss[-1]: del recentss[-1]
      
      self.extra_html_pages[lang] = l = []
      i = 2
      filename = self.html_filenames[lang].rsplit(os.sep, 1)[0]
      if len(self.blog.langs) == 1:
        filename = os.path.join(filename, "ec_recent_posts%s.html")
      else:
        filename = os.path.join(filename, "ec_recent_posts%%s_%s.html" % lang.langs[0])
      for recents in recentss[1:]:
        l.append(HTMLPage(filename % i, docs, lang, "", self.theme, recent_posts = recents, number = i))
        i += 1
      recents = recentss[0]
      
    html_page = HTMLPage(self.html_filenames[lang], docs, lang, rst, self.theme, recent_posts = recents, number = 1)
    return html_page
  
  def build(self):
    archives = []
    for doc in self.pages + self.includes + self.posts + self.subcategories:
      if isinstance(doc, Archives): archives.append(doc)
      else:                         doc.build()
      
    super().build()
    
    for doc in archives: # Archives page need to be built AFTER the index, not before, because it depends on it.
      doc.build()
    
  def save_html_pages(self):
    super().save_html_pages()
    
    if self.need_update and self.html_pages:
      for html_page in self.html_pages.values():
        rss = html_page.get_rss_page()
        f1, f2 = html_page.filename.rsplit(os.sep, 1)
        f2 = "news%s.rss" % f2[5:-5]
        filename = os.path.join(f1, f2)
        write_file(filename, rss)
        

SPECIAL_DOCS = {}
BLOG = None
class Blog(object):
  def __init__(self, blog_dir, langs, number_of_recent_post, ec_multiple_themes, ec_translations, title, author, author_email, url, ec_dont_propagate_posts_for_categories):
    global BLOG
    BLOG                       = self
    self.blog_dir              = blog_dir
    self.langs                 = langs
    self.number_of_recent_post = number_of_recent_post
    self.ec_multiple_themes    = ec_multiple_themes
    self.ec_translations       = ec_translations
    self.docs                  = []
    self.categories_dict       = {}
    self.changed_categories    = []
    self.title                 = title
    self.author                = author
    self.author_email          = author_email
    self.url                   = url
    self.src_filename_2_doc    = {}
    self.pending_links         = defaultdict(list)
    self.ec_dont_propagate_posts_for_categories = ec_dont_propagate_posts_for_categories
    self.search_doc            = None

  def translate(self, txt, lang):
    if isinstance(lang, str):
      return self.ec_translations[(lang, txt)]
    else:
      for l in lang.langs:
        if (l, txt) in self.ec_translations: return self.ec_translations[l, txt]
    
  def scan_files(self, ignored_dirs = set()):
    config_file = os.path.join(self.blog_dir, "conf.py")
    
    for dirpath, dirnames, filenames in os.walk(self.blog_dir):
      if dirpath in ignored_dirs: dirnames[:] = []; continue
      
      for filename in filenames:
        if filename in SPECIAL_DOCS: continue
        full_filename = os.path.join(dirpath, filename)
        if filename.endswith(".rst") or filename.endswith(".inc"): self.create_doc(full_filename)
        
    # Put archives at the end
    archives = [doc for doc in self.docs if isinstance(doc, Archives)]
    for doc in archives: self.docs.remove(doc)
    self.docs.extend(archives)
    
  def create_doc(self, src_filename):
    if os.path.islink(src_filename):
      link_target = os.readlink(src_filename)
      if link_target.startswith("."): link_target = os.path.join(os.path.dirname(src_filename), link_target)
      link_target = os.path.normpath(link_target)
      if link_target in self.src_filename_2_doc: self.src_filename_2_doc[link_target].add_link(src_filename)
      else:                                      self.pending_links[link_target].append(src_filename)
      return
    
    date = is_post(src_filename)
    doc  = None
    if   src_filename.endswith("_comments.inc"): doc = CommentsInclude(self, src_filename)
    elif src_filename.endswith(".inc"):          doc = Include        (self, src_filename) # Must be checked before posts, for comments includes
    elif date:                                   doc = Post           (self, src_filename, date)
    elif src_filename.endswith("archives.rst"):  doc = Archives       (self, src_filename)
    elif src_filename.endswith("search.rst"):    doc = Search         (self, src_filename)
    elif src_filename.endswith("index.rst"):     doc = Category       (self, src_filename)
    else:                                        doc = Page           (self, src_filename)
    if doc: self.src_filename_2_doc[src_filename] = doc
    
    if src_filename in self.pending_links:
      for pending_link in self.pending_links[src_filename]: doc.add_link(pending_link)
      del self.pending_links[src_filename]
    
  def ready(self):
    for doc in self.docs: doc.ready()
    themes = set()
    for doc in self.docs:
      theme = doc.theme
      while theme:
        themes.add(theme)
        theme = theme.parent
    for theme in themes: theme.symlink_files(self.blog_dir)

  def check_update(self, force = False):
    self.categories_dict[""].check_update(force)
    
  def build(self):
    self.categories_dict[""].build()
    
  def save_html_pages(self):
    for doc in self.docs: doc.save_html_pages()
    
  def build_search(self, force = False):
    start_char = len(self.blog_dir) + 5 # 5 is for /html
    if not self.blog_dir.endswith(os.sep): start_char += 1
    
    # Use 2 db because Sqlite does not accept to remove/update from content-less table
    
    import sqlite3
    for lang in self.langs:
      db_filename0 = os.path.join(self.blog_dir, "searchdb%s.sqlite3"    % lang.get_suffix())
      if force:
        try: os.unlink(db_filename0)
        except: pass
        new_db = True
      else:
        new_db = not os.path.exists(db_filename0)
        
      db0 = sqlite3.connect(db_filename0)
      cursor0 = db0.cursor()
      if new_db:
        cursor0.execute("""CREATE TABLE page(id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT, title TEXT, text TEXT)""")
      for doc in self.docs:
        if not doc.need_update: continue
        page = doc.get_html_page(lang)
        if page:
          title = page.get_title()
          text = page.get_text()
          text = " ".join(text.split())
          text = "%s %s" % (title, text)
          page_name = page.filename[start_char:]
          if new_db:
            page_id = None
          else:
            page_id = cursor0.execute("""SELECT id FROM page WHERE name=?""", (page_name,)).fetchone()
            if not page_id is None:
              page_id = page_id[0]
              cursor0.execute("""UPDATE page SET title=?, text=? WHERE id=?""", (title, text, page_id,))
          if page_id is None:
            cursor0.execute("""INSERT INTO page VALUES (NULL, ?, ?, ?)""", (page_name, title, text))
            #page_id = cursor.lastrowid
      db0.commit()
      data = list(cursor0.execute("""SELECT id, name, title, text FROM page"""))
      
      db_filename = os.path.join(self.blog_dir, "html", "searchindex%s.sqlite3" % lang.get_suffix())
      try: os.unlink(db_filename)
      except: pass
      db  = sqlite3.connect(db_filename)
      cursor      = db.cursor()
      cursor.execute("""PRAGMA page_size=1024""")
      cursor.execute("""CREATE TABLE page(id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT, title TEXT)""")
      cursor.execute("""CREATE VIRTUAL TABLE fts USING fts4(content="", text, matchinfo=fts3)""")
      db.executemany("INSERT INTO page VALUES (?,?,?)", [(page_id, page_name, title) for (page_id, page_name, title, text) in data])
      db.executemany("INSERT INTO fts(docid, text) VALUES (?,?)", [(page_id, text) for (page_id, page_name, title, text) in data])
      cursor.execute("""INSERT INTO fts(fts) VALUES('optimize')""")
      db.commit()
      cursor0.close()
      cursor.close()
      db.execute("""VACUUM;""")
      db0.close()
      db.close()
      
      
def is_post(src_filename):  
  try:
    year, month, day, title = os.path.basename(src_filename).split("_", 3)
    return Date(int(year), int(month), int(day))
  except: return None

def symlink(src, dest):
  if os.path.lexists(dest):
    if os.path.islink(dest):
      dest_link = os.path.join(os.path.dirname(dest), os.readlink(dest))
      if os.path.samefile(dest_link, src): return
    os.unlink(dest)
  print("link  %s" % dest)
  os.symlink(src, dest)

def do(s):
  print("\n%s" % s)
  if os.system(s) != 0:
    print("\nÉCHEC!")
    sys.exit()

LANGS = {}
class _Language(object):
  def __init__(self, name, locale, *langs):
    self.name     = name
    self.locale   = locale
    self.langs    = langs
    self.priority = len(LANGS) + 1
    LANGS[langs[0]] = self
    
  def __repr__(self): return "<Language '%s'>" % self.langs[0]
  
  def set_locale_current(self):
    locale.setlocale(locale.LC_ALL, self.locale)
    
  def get_suffix(self):
    if len(LANGS) == 1: return ""
    return "_%s" % self.langs[0]
  
def Language(name, locale, *langs):
  if langs[0] in LANGS: return LANGS[langs[0]]
  return _Language(name, locale, *langs)

AVAILABLES_DOC_LANGS = {}
def get_available_doc_langs(doc_name):
  if not doc_name in AVAILABLES_DOC_LANGS:
    langs = set()
    src_filename = os.path.join(BLOG.blog_dir, "%s.rst" % doc_name)
    if os.path.exists(src_filename):
      print("readlang %s" % src_filename)
      s = open(src_filename).read()
      for line in s.split("\n"):
        if line.startswith(".. lang::"):
          lang = LANGS.get(line[len(".. lang::"):].strip())
          if lang: langs.add(lang)
    else: langs = LANGS.values() # Search page, etc
    AVAILABLES_DOC_LANGS[doc_name] = sorted(langs, key = lambda x: x.priority)
  return AVAILABLES_DOC_LANGS[doc_name]


from eclaircie.rst   import *
from eclaircie.theme import *
from eclaircie.page  import *


BLOG = None
EC_TRANSLATIONS = None
EC_HAS_COMMENTS = False
def run(conf_filename, force = False):
  global EC_TRANSLATIONS, EC_HAS_COMMENTS, BLOG
  d = {}
  s = open(conf_filename).read().split("\n")
  s = "\n".join(l for l in s if not l.startswith(".. lang::"))
  exec(s, d)
  blog_dir                 = os.path.dirname(conf_filename)
  number_of_recent_post    = d["number_of_recent_post"]
  EC_TRANSLATIONS          = d["ec_translations"]
  title                    = d["project"]
  author                   = d["author"]
  author_email             = d["author_email"]
  url                      = d["url"]
  comments_mail_dir        = d["comments_mail_dir"]
  gallery_miniature_width  = d["gallery_miniature_width"]
  gallery_miniature_height = d["gallery_miniature_height"]
  gallery_import_width     = d["gallery_import_width"]
  gallery_import_height    = d["gallery_import_height"]
  ec_translations          = d["ec_translations"]
  ec_dont_propagate_posts_for_categories = d["ec_dont_propagate_posts_for_categories"]
  
  os.makedirs(os.path.join(blog_dir, "html"), exist_ok = True)
  
  parse_theme_dir(os.path.join(os.path.dirname(__file__), "themes"))
  
  ec_multiple_themes = d.get("ec_multiple_themes") or {}
  ec_multiple_themes[""] = d.get("html_theme", "ec_base")
  ec_multiple_themes = [ (path, THEMES[theme_name]) for path, theme_name in ec_multiple_themes.items() ]
  ec_multiple_themes.sort(key = lambda category_theme: len(category_theme[0]), reverse = True)
  
  if comments_mail_dir:
    EC_HAS_COMMENTS = True
    import eclaircie.comments
    comments_changed_doc_names = eclaircie.comments.update_comments(blog_dir, title, EC_TRANSLATIONS, comments_mail_dir, force)
    
  symlink(os.path.join(blog_dir, "_static"),    os.path.join(blog_dir, "html", "_static"))
  symlink(os.path.join(blog_dir, "_images"),    os.path.join(blog_dir, "html", "_images"))
  symlink(os.path.join(blog_dir, "_downloads"), os.path.join(blog_dir, "html", "_downloads"))
  
  for dirpath, dirnames, filenames in os.walk(os.path.join(blog_dir, "_images")):
    for filename in filenames:
      if filename.startswith("miniature_"): continue
      if filename.startswith("youtube_preview_"): continue
      reduce_image(os.path.join(dirpath, filename), os.path.join(dirpath, "miniature_%s" % filename), gallery_miniature_width, gallery_miniature_height)
      
  langs = sorted(LANGS.values(), key = lambda lang: 0 if lang.langs[0] == "en" else 1) # Start by English
  BLOG = Blog(blog_dir, langs, number_of_recent_post, ec_multiple_themes, ec_translations, title, author, author_email, url, ec_dont_propagate_posts_for_categories)
    
  ignored_dirs = {
    os.path.join(blog_dir, "html"), os.path.join(blog_dir, "_static"),
    os.path.join(blog_dir, "_images"), os.path.join(blog_dir, "_downloads")
  }
  
  for dirpath, dirnames, filenames in os.walk(os.path.join(blog_dir)):
    if dirpath in ignored_dirs: dirnames[:] = []; continue
    for filename in filenames:
      if filename.endswith(".py") and (filename != "conf.py"):
        filename = os.path.join(dirpath, filename)
        print("running  %s..." % filename)
        py = open(filename).read()
        exec(py, { "__file__" : filename } )
        
  BLOG.scan_files(ignored_dirs)
  BLOG.ready()
  BLOG.check_update(force)
  BLOG.build()
  BLOG.save_html_pages()
  BLOG.build_search(force)
  
  if len(LANGS) > 1:
    index_filename = os.path.join(blog_dir, "html", "index.html")
    if not os.path.exists(index_filename):
      s = """<?xml version="1.0" encoding="utf-8" ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<body>

<script language="javascript"><!--
var language = window.navigator.userLanguage || window.navigator.language;
language = language.slice(0,2);
"""
      for lang in LANGS.values():
        s += """if (language == "%s") document.location.href="index_%s.html";\n""" % (lang.langs[0], lang.langs[0])
      s += """else document.location.href="index_en.html";
//--></script>
<p>This site is available in several languages:</p>\n"""
      for lang in LANGS.values():
        s += """<a href="index_%s.html">%s</a><br/>\n""" % (lang.langs[0], lang.name)
      s += """</body></html>"""
      open(index_filename, "w").write(s)
    



#!/usr/bin/env python3

import sys, os, datetime
from eclaircie import *

langs = [
    Language("Français", "fr_FR.utf8", "fr", "en"),
    Language("English",  "en_US.utf8", "en"),
]


# The number of recent posts displayed per page

number_of_recent_post = 5

# The default theme.
# Per-category themes can be specified in themes.conf

html_theme = 'green_roses'

# This list allows to specify per-category/document themes (overriding the default theme above).

ec_multiple_themes = {
  "themes/champi" : "champi",
  "themes/green_roses" : "green_roses",
  "themes/balazar" : "balazar",
# "category/subcategory" : "theme",
}


# General information about the project.
# Note: you can use '.. lang::' directive here if you want a language-dependent website's title

project      = 'éClaircie' # This is the website's title
author       = 'Jean-Baptiste "Jiba" Lamy'
author_email = "your@email.here"
copyright    = '2014, %s' % author
url          = "http://xxx.org" # This is the final address of the website, online

# Translation strings ; you may want to personalize them or to add additional languages.

year = datetime.date.today().year

ec_translations = {
  ("fr", "older_posts") : "messages plus anciens >",
  ("en", "older_posts") : "older entries >",
  ("fr", "newer_posts") : "< messages plus récents",
  ("en", "newer_posts") : "< newer entries",
  ("fr", "in_category") : "dans",
  ("en", "in_category") : "in",
  ("fr", "more") : "(Suite et commentaires...)",
  ("en", "more") : "(More and comments...)",
  ("fr", "comment") : "(Voir les commentaires...)",
  ("en", "comment") : "(View comments...)",
  ("fr", "comment_title") : "Commentaires",
  ("en", "comment_title") : "Comments",
  ("fr", "add_comment") : "(Ajouter un commentaire - par mail, protège votre vie privée !)",
  ("en", "add_comment") : "(Add a comment - by mail, protect your privacy!)",
  ("fr", "comment_by") : "**Commentaire de %s** (%s)",
  ("en", "comment_by") : "**Comment by %s** (%s)",
  ("fr", "search") : "Recherche",
  ("en", "search") : "Search",
  ("fr", "search_result") : "Résultat de la recherche",
  ("en", "search_result") : "Search results for",
  ("fr", "search_no_result") : "Pas de résultat.",
  ("en", "search_no_result") : "No result.",
  ("fr", "bottom") : """© Copyright %s, Jean-Baptiste "Jiba" Lamy. Propulsé par <a href="http://www.lesfleursdunormal.fr/static/informatique/eclaircie/index_fr.html">éClaircie</a>, le moteur de blog statique et sans nuage !""" % year,
  ("en", "bottom") : """© Copyright %s, Jean-Baptiste "Jiba" Lamy. Powered by <a href="http://www.lesfleursdunormal.fr/static/informatique/eclaircie/index_fr.html">éClaircie</a>, the static cloud-less blog engine!""" % year,
}

# The mail directory where comments mails are located.
# When receiving mails, comment mails can be detected with their "To" header (which starts with the
# website name, i.e. the value of the 'project' option above). They should be moved into a specific folder.
# 
# Set to None to disable comments by mail.

comments_mail_dir = "/home/jiba/mail/blog_comments"
#comments_mail_dir = None

# Maximum size for miniature in image gallery

gallery_miniature_width  = 200
gallery_miniature_height = 150

# Maximum size when importing images in an image gallery (i.e. for images not already located in /_images/)
# Bigger image will be reduced

gallery_import_width     = 1400
gallery_import_height    = 1050


# The following set can be used to tag some categories as "special" and make then not propagate
# their post to the parent caegory.
# Note: the values must be "full" category name, including "/index" at the end,
# for example: "informatique/eclaircie/example_site/index"

ec_dont_propagate_posts_for_categories = set()







import sys, os, re, io
import docutils.writers.html5_polyglot as html_writer
#import docutils.writers.html4css1 as html_writer
import docutils.core, docutils.nodes
from docutils.parsers.rst import roles

import eclaircie

class MyTranslator(html_writer.HTMLTranslator):
  def __init__(self, document):
    super().__init__(document)
    
  def visit_image(self, node):
    super().visit_image(node)
    self.body[-1] = self.body[-1].replace('"/_images', '"__BLOG_HTML_ROOT__/_images', )
    
  def visit_title_reference(self, node):
    splitted = node.children[0].split("<", 1)
    if len(splitted) == 2:
      label, ref = splitted
      ref = ref.rsplit(">", 1)[0]
    else:
      label = ""
      ref = splitted[0]

    if "#" in ref:
      ref, anchor = ref.split("#", 1)
      anchor = "#%s" % anchor
    else:
      anchor = ""
      
    if ref.startswith("/"):
      src_filename = os.path.join(eclaircie.BLOG.blog_dir, ref[1:])
    else:
      src_filename = os.path.join(eclaircie.BLOG.blog_dir, CURRENT_PATH, ref)
    src_filename = os.path.abspath(src_filename)
    
    doc = eclaircie.BLOG.src_filename_2_doc.get(src_filename + ".rst")
    if not doc:
      doc = eclaircie.BLOG.src_filename_2_doc.get(src_filename + "/index.rst")
      if not doc:
        print(src_filename)
        raise ValueError
    ref = doc.doc_name
    
    if not label.strip():
      if len(eclaircie.BLOG.langs) == 1:
        lang = eclaircie.BLOG.langs[0].langs[0]
      else:
        lang = CURRENT_LANG_PREFIX[1:]
      label = doc.get_html_page(eclaircie.LANGS[lang]).get_title()
    
    node.children[0] = docutils.nodes.Text(label.strip())
    self.body.append(self.starttag(node, 'a', href = "__BLOG_HTML_ROOT__/%s%s.html%s" % (ref, CURRENT_LANG_PREFIX, anchor), CLASS = "reference internal"))
    
  def depart_title_reference(self, node):
    self.body.append('</a>')
    
  def visit_DownloadNode(self, node):
    splitted = node.children[0].split("<", 1)
    if len(splitted) == 2:
      label, ref = splitted
      ref = ref.rsplit(">", 1)[0]
    else:
      label = ""
      ref = splitted[0]
      
    node.children[0] = docutils.nodes.Text((label or ref.rsplit(os.sep, 1)[1]).strip())
    self.body.append(self.starttag(node, 'a', href = "__BLOG_HTML_ROOT__%s" % ref, CLASS = "download reference internal"))
    
  def depart_DownloadNode(self, node):
    self.body.append('</a>')
    
  def visit_literal_block(self, node):
    self.body.append(self.starttag(node, 'pre', '', CLASS = 'literal-block'))
    if 'code' in node.get('classes', []):
      self.body.append('<code>')
      
  def depart_literal_block(self, node):
    if 'code' in node.get('classes', []):
      self.body.append('</code>')
    self.body.append('</pre>\n')
    
  def visit_system_message(self, node):
    #print(" ".join(node.traverse(docutils.nodes.Text)))
    node.children = []
    
  def depart_system_message(self, node):
    pass
  
writer = html_writer.Writer()
writer.translator_class = MyTranslator

def doc_role(name, rawtext, text, lineno, inliner, options = {}, content = []):
  node = docutils.nodes.title_reference(rawtext, text, **options)
  return [node], []
roles.register_canonical_role("doc", doc_role)

class DownloadNode(docutils.nodes.Inline, docutils.nodes.TextElement): pass

def download_role(name, rawtext, text, lineno, inliner, options = {}, content = []):
  node = DownloadNode(rawtext, text, **options)
  return [node], []
roles.register_canonical_role("download", download_role)

CURRENT_LANG_PREFIX = ""
CURRENT_PATH = ""

title_regexp = re.compile(r"</?h[1-9]>")

def fix_titles(html):
  nb = 0
  def rep(match):
    nonlocal nb
    nb += 1
    if nb >= 2:
      h = match.group()
      level = int(h[-2])
      return "%s%s>" % (h[:-2], level + 1)
    return match.group()
  return title_regexp.sub(rep, html)

def shift_titles(html, offset = 1):
  def rep(match):
    h = match.group()
    level = int(h[-2])
    return "%s%s>" % (h[:-2], max(level + offset, 0))
  return title_regexp.sub(rep, html)
  
  
def rst_2_html(rst, lang, path, orig_file):
  global CURRENT_LANG_PREFIX, CURRENT_PATH
  if lang: CURRENT_LANG_PREFIX = "_%s" % lang
  else:    CURRENT_LANG_PREFIX = ""
  CURRENT_PATH = path
  
  ##html = docutils.core.publish_string(rst, writer = writer).decode("utf8")
  #parts = docutils.core.publish_parts(source = rst, writer_name = "html")
  
  sys.stderr = buf = io.StringIO()
  parts = docutils.core.publish_parts(source = rst, writer = writer)
  sys.stderr = sys.__stderr__
  err = buf.getvalue()
  if err:
    print("WARNING: RST error in:", orig_file)
    print(err)
  html = parts["body_pre_docinfo"] + parts["fragment"]
  return fix_titles(html)



if __name__ == "__main__":
  rst = """
Grand titre
===========

bla bla bla...

Petit titre
***********

bla bla bla...

Mini titre
----------

bla bla bla...

Mini titre
----------

bla bla bla...

"""
  print(rst_2_html(rst, "", "", ""))
  



# éClaircie
# Copyright (C) 2014-2018 Jean-Baptiste LAMY

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os, locale
import email, email.header, email.utils
from datetime import date as Date
from datetime import datetime as DateTime

def decode_header(header): return str(email.header.make_header(email.header.decode_header(header)))

def update_comments(blog_dir, blog_name, translations, mail_dir, force = False):
  blog_name            = blog_name.replace(" ", "")
  last_update_filename = os.path.join(blog_dir, "last_comments_update")
  if (not force) and os.path.exists(last_update_filename): last_update_date = os.path.getmtime(last_update_filename)
  else:                                                    last_update_date = None
  filenames = [filename for filename in os.listdir(mail_dir) if not filename.startswith(".")]
  filenames.sort(key = lambda a: int(a))
  
  doc_name_2_comments = {}
  for filename in filenames:
    filename = os.path.join(mail_dir, filename)
    if last_update_date and (last_update_date > os.path.getmtime(filename)): continue
    
    print("comment  %s" % filename)
    doc_name, comments = parse_mail(blog_name, translations, filename)
    
    if doc_name:
      doc_name_2_comments[doc_name] = doc_name_2_comments.get(doc_name, "") + comments
  
  changed_doc_names = set()
  for doc_name, comments in doc_name_2_comments.items():
    changed_doc_names.add(doc_name)
    if last_update_date:
      f = open(os.path.join(blog_dir, doc_name[1:]) + "_comments.inc", "a")
    else:
      f = open(os.path.join(blog_dir, doc_name[1:]) + "_comments.inc", "w")
    f.write(comments)
  
  open(last_update_filename, "w")
  return changed_doc_names



def parse_mail(blog_name, translations, filename):
  mail     = email.message_from_file(open(filename))
  dest     = decode_header(mail["To"]).strip()
  if dest.startswith("'") or dest.startswith('"'): dest = dest[1:]
  if not dest.startswith(blog_name.replace(" ", "")):
    print("WARNING: non-comment email %s !" % filename)
    return None, None
  doc_name = dest[len(blog_name) + 1:].split()[0]
  doc_name, lang = doc_name.rsplit("_", 1)
  lang = lang[:2] # lang may ends with " or '
  
  address  = decode_header(mail["From"])
  if   "<" in address:
    sender = address.split("<")[0].strip().replace('"', "")
    if sender.startswith("'"): sender = sender.replace("'", "")
  elif address.count('"') >= 2:
    sender = address.split('"')[1]
  elif address.count("'") >= 2:
    sender = address.split("'")[1]
  else:
    sender = address.split("@")[0]

  datetime = email.utils.parsedate_to_datetime(mail["Date"])    
  text     = read_content(mail)
  
  import eclaircie
  locale.setlocale(locale.LC_ALL, eclaircie.LANGS[lang].locale)
  
  lang2 = lang
  #if lang == "en": lang2 = "all" # Hack in order to have english comment on french pages
  #else:            lang2 = lang
  
  return doc_name, """
.. lang:: %s

.. container:: comments

   .. container:: comments-title

      %s

   %s
""" % (lang2, translations[(lang, "comment_by")] % (sender, datetime.strftime("%c").strip()), text.replace("\n", "\n   "))


def read_content(mail):
  content_type = mail.get_content_type()
  if mail.is_multipart():
    for submail in mail.get_payload():
      text = read_content(submail)
      if not text is None: return text
      
  elif content_type == "text/plain":
    try:
      return mail.get_payload(decode = True).decode(mail.get_content_charset() or "latin")
    except UnicodeDecodeError:
      return mail.get_payload(decode = False)
    






# éClaircie
# Copyright (C) 2018 Jean-Baptiste LAMY

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os, re, locale, datetime
from html import escape
from urllib.parse import urljoin

from eclaircie import Post, Category, Page, LANGS
from eclaircie.rst import rst_2_html
from eclaircie.email_obfuscator import email_link

h1 = re.compile("""<h1.*?>(.*?)</h1>\n?""")
h2 = re.compile("""<h2.*?>(.*?)</h2>\n?""")

script_regexp = re.compile("""<script.*?</script>""")
to_text = re.compile("""<.*?>""")

def split_lang(rst):
  lang_lines = []
  available_langs = set()
  for line in rst.split("\n"):
    if line.startswith(".. lang::"):
      lang = line[len(".. lang::"):].strip()
      available_langs.add(lang)
      lang_lines.append((lang, []))
    else:
      if not lang_lines: lang_lines.append(("all", []))
      lang_lines[-1][1].append(line)
      
  available_langs.discard("all")
  if not available_langs: available_langs = set(LANGS.keys())
  
  r = {}
  for lang in LANGS.values():
    for included_lang in lang.langs:
      if included_lang in available_langs:
        result_lines = []
        for l, lines in lang_lines:
          if (l == included_lang) or (l == "all"): result_lines.extend(lines)
        r[lang] = "\n".join(result_lines)
        break
      
  return r


class HTMLPage(object):
  def __init__(self, filename, docs, lang, rst, theme, **kargs):
    self.filename  = filename
    self.docs      = docs
    self.doc       = docs[-1]
    self.lang      = lang
    self.rst       = rst
    self.theme     = theme
    self.title     = None
    self.html      = None
    self.html_page = None
    self.has_two_part = False
    self.__dict__.update(kargs)

  def __repr__(self): return "<HTMLPage '%s'>" % self.filename

  def get_url(self):
    return os.path.relpath(self.filename, os.path.join(self.doc.blog.blog_dir, "html"))
  
  def get_hyperlink(self, label = ""):
    return """<a href="__BLOG_HTML_ROOT__/%s">%s</a>""" % (self.get_url(), label or self.get_title())
  
  def get_title(self):
    if self.title is None:
      if len(self.doc.blog.langs) == 1: lang = ""
      else:                             lang = self.lang.langs[0]
      
      path = os.path.relpath(os.path.dirname(self.filename), os.path.join(self.doc.blog.blog_dir, "html"))
      
      if "\n.. more::" in self.rst:
        self.has_two_part = True
        rst_first_part, rst_second_part = self.rst.split("\n.. more::", 1)
        self.html = rst_2_html("\n".join([rst_first_part, rst_second_part]), lang, path, self.filename)
        self.html_first_part = rst_2_html(rst_first_part, lang, path, self.filename)
      else:
        self.html = self.html_first_part = rst_2_html(self.rst, lang, path, self.filename)
        
      self.title = ""
      if isinstance(self.doc, Post):
        def rep(match):
          self.title = match.group(1)
          return "<h1>%s</h1> " % self.get_hyperlink()
        self.html            = h1.sub(rep, self.html)
        self.html_first_part = h1.sub(rep, self.html_first_part)
      else:
        def rep(match):
          self.title = match.group(1)
          return ""
        self.html = h1.sub(rep, self.html)
    return self.title
  
  def get_html(self):
    if self.html is None: self.get_title()
    if isinstance(self.doc, Post):
      if self.comments:
        if len(self.doc.blog.langs) == 1: lang = ""
        else:                             lang = self.lang.langs[0]
        path = os.path.relpath(os.path.dirname(self.filename), os.path.join(self.doc.blog.blog_dir, "html"))
        self.html += "%s\n" % rst_2_html(self.comments, lang, path, self.filename)
        self.html += "%s\n" % self.get_add_comment_link()
      else:
        self.html += "%s\n" % self.get_add_comment_link()
    return self.html
  
  def get_html_first_part(self):
    if self.html is None: self.get_title()
    if self.has_two_part:
      return self.html_first_part + "\n%s" % self.get_hyperlink(label = self.doc.blog.translate("more", self.lang))
    else:
      if self.comments:
        return self.html_first_part + "\n%s" % self.get_hyperlink(label = self.doc.blog.translate("comment", self.lang))
      else:
        return self.html_first_part + "\n%s" % self.get_add_comment_link()
      
  def get_add_comment_link(self):
    return email_link(self.doc.blog.author_email,
                      "%s /%s_%s" % (self.doc.blog.title.replace(" ", ""), self.doc.doc_name, self.lang.langs[0]),
                      self.doc.blog.translate("add_comment", self.lang),
                      localurl = "file://%s" % self.doc.blog.blog_dir,
                      url      = self.doc.blog.url)
  
  def get_html_page(self):
    if self.html_page is None:
      pages = [doc.get_html_pages()[self.lang] for doc in self.docs]
      self.html_page = self.theme.htmlize(pages, self, self.lang)
    return self.html_page
  
  def get_rss_page(self):
    root_category = self.doc.blog.categories_dict[""]
    title = root_category.get_html_page(self.lang).get_title()
    if not root_category is self.doc:
      title = "%s — %s" % (title, self.get_title())
      
    locale.setlocale(locale.LC_ALL, "C") # For strftime(); RSS need dates in english :-(
    date = datetime.date.today().strftime("%a, %d %b %Y %H:%M:%S +0000")
    
    rss = """<?xml version="1.0" encoding="utf-8"?>
<rss version="2.0"><channel>
<title>%s</title>
<link>%s</link>
<description></description>
<language>%s</language>
<pubDate>%s</pubDate>
""" % (title, urljoin(self.doc.blog.url, self.get_url()), self.lang.langs[0], date)
    
    for post in self.recent_posts:
      post_page = post.get_html_page(self.lang)
      url = urljoin(self.doc.blog.url, post_page.get_url())
      html = post_page.get_html_first_part()
      #html += """"""
      rss += """<item>\n"""
      rss += """  <link>%s</link>\n""" % url
      rss += """  <guid>%s</guid>\n""" % url
      rss += """  <title><![CDATA[%s]]></title>\n""" % post_page.get_title()
      rss += """  <description><![CDATA[%s]]></description>\n""" % html
      rss += """  <category><![CDATA[%s]]></category>\n""" % " ".join(category.doc_name for category in post.categories)
      rss += """  <pubDate>%s</pubDate>\n""" % post.date.strftime("%a, %d %b %Y %H:%M:%S +0000")
      rss += """</item>\n"""
      
    rss += """</channel></rss>"""
    return rss
  
  def get_text(self):
    html = self.get_html()
    text = script_regexp.sub(" ", html)
    return to_text.sub(" ", text)
  



# éClaircie
# Copyright (C) 2018 Jean-Baptiste LAMY

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os, re, locale
from html import escape

from eclaircie import Post, Category, Page, Archives, Search
from eclaircie.email_obfuscator import email_link
from eclaircie.rst import shift_titles

h1 = re.compile('''<h1.*?>(.*?)</h1>\n?''')

THEMES = {}
def parse_theme_dir(dir):
  for subdir in os.listdir(dir):
    filename = os.path.join(dir, subdir, "theme.conf")
    if os.path.exists(filename):
      theme = THEMES.get(subdir) or Theme(subdir)
      theme.load(filename)

_CACHE = {}

class Theme(object):
  def __init__(self, name):
    self.name = name
    self.filename = ""
    self.parent = None
    self.stylesheets = []
    self.javascripts = []
    self.pygments_style = "sphinx"
    THEMES[self.name] = self
    
  def __repr__(self): return "<Theme '%s'>" % self.name
    
  def load(self, filename):
    self.filename = filename
    for line in open(filename).read().split("\n"):
      if (not line) or (line[0] == "#") or (line[0] == "["): continue
      option, value = line.split("=", 1)
      option = option.strip()
      value  = value .strip()
      if   option == "inherit": self.parent = THEMES.get(value) or Theme(value)
      elif option == "pygments_style": self.pygments_style = value
      elif option == "stylesheet": self.stylesheets.append(value)
      elif option == "javascript": self.javascripts.append(value)
      else:
        raise ValueError("Unknown option in theme config file: '%s' in '%s'!" % (line, filename))
      
  def get_stylesheets(self):
    if self.parent: return self.parent.get_stylesheets() + self.stylesheets
    else:           return self.stylesheets
    
  def get_javascripts(self):
    if self.parent: return self.parent.get_javascripts() + self.javascripts
    else:           return self.javascripts
    
  def symlink_files(self, blog_dir):
    if not self.filename: return
    from eclaircie import symlink
    static_dir = os.path.join(os.path.dirname(self.filename), "static")
    for filename in os.listdir(static_dir):
      full_filename = os.path.join(static_dir, filename)
      symlink(full_filename, os.path.join(blog_dir, "_static", filename))
      
  def perspective_title(self, pages, page):
    html = ""
    is_post = isinstance(pages[-1].doc, Post)
    if is_post: pages2 = pages[:-1]
    else:       pages2 = pages
    for i in range(len(pages2) - 1):
      html += """<span style="font-size: smaller;">"""
    for page2 in pages2:
      if page2 is pages2[-1]:
        if (not is_post) and (page2 is page):
          html += """%s""" % page2.get_title()
        else:
          html += """%s&nbsp;/""" % page2.get_hyperlink()
      else:
        html += """%s&nbsp;/</span> """ % page2.get_hyperlink()
        
    html = """<div class="related1 related"><div class="related1wrapper">%s</div></div>""" % html
    return html
  
  def side_bar(self, pages, lang):
    icons = ["""<div class="links"><span class="link-lang">"""]
    html_pages = pages[-1].doc.get_html_pages()
    for lang2 in sorted(pages[0].doc.blog.langs, key = lambda l: l.priority):
      page = html_pages.get(lang2)
      if page:
        filename = os.path.abspath(pages[-1].filename)[:-7].rsplit(os.sep, 1)[-1]
        icons.append("""<a class="link-lang" href="%s%s.html" title="%s"><img src="__BLOG_HTML_ROOT__/_static/lang_%s.svg"></a>""" % (filename, lang2.langs[0], lang2.name, lang2.langs[0]))
    if len(pages[0].doc.blog.langs) == 1:
      rss_lang = ""
    else:
      rss_lang = "_%s" % pages[-1].lang.langs[0]
    icons.append("""<a class="link-rss" href="news%s.rss" title="RSS"><img src="__BLOG_HTML_ROOT__/_static/rss.svg"></a>""" % rss_lang)
    icons.append("""</span></div>""")
    
    htmls = ["""<h3>%s</h3>""" % pages[0].get_hyperlink()]
    
    doc_2_status = {}
    for page in pages: doc_2_status[page] = 1 # expanded
    if isinstance(pages[-1].doc, Post): doc_2_status[pages[-2]] = 2 # current
    else:                               doc_2_status[pages[-1]] = 2 # current
    
    self._side_bar_category(htmls, pages[0], doc_2_status, lang)
    
    if pages[0].doc.blog.search_doc:
      search_doc  = pages[0].doc.blog.search_doc
      search_page = search_doc.get_html_page(lang)
      search = """<div id="searchbox">
  <h3>%s</h3>
    <form class="search" action="__BLOG_HTML_ROOT__/%s" method="get">
      <input type="text" name="q"/>
    </form>
</div>
""" % (search_page.get_hyperlink(), search_page.get_url())
    else:
      search = ""
    
    html = """%s
<div class="sphinxsidebar" role="navigation" aria-label="main navigation"><div class="sphinxsidebarwrapper">
%s
%s
<h3>eMail</h3>
%s
</div></div>""" % ("\n".join(icons),
                   "\n".join(htmls),
                   search,
                   email_link(pages[0].doc.blog.author_email))
    return html
  
  def _side_bar_category(self, htmls, page_category, doc_2_status, lang):
    htmls.append("<ul>")
    subs = [doc.get_html_page(lang) for doc in page_category.doc.subcategories + page_category.doc.pages]
    subs = [page for page in subs if page]
    subs.sort(key = lambda page: locale.strxfrm(page.get_title().lower()))
    for page in subs:
      if isinstance(page.doc, Search): continue
      status = doc_2_status.get(page, 0)
      if status == 2: 
        htmls.append("""<li class="toctree-l1 current">%s</li>""" % page.get_hyperlink())
      else:
        htmls.append("""<li class="toctree-l1">%s</li>""" % page.get_hyperlink())
      if status and isinstance(page.doc, Category):
        self._side_bar_category(htmls, page, doc_2_status, lang)
    htmls.append("</ul>")
    
  def nav_bar(self, pages, page):
    items = []
    
    if page.lang in page.doc.extra_html_pages:
      pages = [page.doc.get_html_page(page.lang)]
      pages.extend(page.doc.extra_html_pages[page.lang])

      if not page is pages[0]:
        items.append(pages[page.number - 2].get_hyperlink(page.doc.blog.ec_translations[(page.lang.langs[0], "newer_posts")]))
        
      for p in pages:
        if p is page:
          items.append(str(p.number))
        else:
          items.append(p.get_hyperlink(str(p.number)))

      if not page is pages[-1]:
        items.append(pages[page.number].get_hyperlink(page.doc.blog.ec_translations[(page.lang.langs[0], "older_posts")]))
        
    archives_filename = os.path.join(os.path.dirname(page.doc.src_filename), "archives.rst")
    archives = page.doc.blog.src_filename_2_doc.get(archives_filename)
    if archives:
      items.append(archives.get_html_page(page.lang).get_hyperlink())
      
    if not items: return ""
    return """<div class="nav_bar">\n%s\n</div>\n""" % " | ".join(items)
  
  def archives(self, pages, lang):
    doc = pages[-1].doc
    html = ""
    posts = [post.get_html_page(lang) for post in doc.blog.docs if isinstance(post, Post)]
    posts = [post for post in posts if post]
    posts.sort(key = lambda post: post.doc.date)
    
    year  = None
    month = None
    
    for post in posts:
      if post.doc.date.year != year:
        if month: html += """</ul>\n"""
        year  = post.doc.date.year
        month = None
        html += """<h2>%s</h2>\n""" % year
      if post.doc.date.month != month:
        if month: html += """</ul>\n"""
        month = post.doc.date.month
        html += """<h3>%s %s</h3>\n""" % (post.doc.date.strftime("%B"), year)
        html += """<ul>\n"""
        
      html += """<li>%s %s</li>\n""" % (post.doc.date.strftime("%x"), post.get_hyperlink())
      
    if month: html += """</ul>\n"""
    return html
  
  def search(self, pages, lang):
    html = r"""
<br/>
<input type="text" name="q" id="search_input"/><input type="submit" value="Ok" onclick="on_search();"/>
<div id="search_results" class="search_results"></div>
<script>
var q = decodeURIComponent(window.location.search.substring(3));

var db;
var xmlhttp = new XMLHttpRequest();
xmlhttp.onreadystatechange = function(){
  if(xmlhttp.status == 200 && xmlhttp.readyState == 4) {
    var data = new Uint8Array(this.response);
    db = new SQL.Database(data);
    if (q != "") { search(q); }
  }
};
xmlhttp.open("GET", "searchindex_%s.sqlite3", true);
xmlhttp.responseType = "arraybuffer";
xmlhttp.send();

function search(query) {
  query = query.replace("+", " ");
  var keywords = query.split(" ");
  var keywords2 = "";
  for (k in keywords) { keywords2 = keywords2 + keywords[k] + "* "; }
  
  var input = document.getElementById("search_input");
  if (input.value != query) input.value = query;
  
  var results_div = document.getElementById("search_results");
  results_div.innerHTML = "...";
  var rs = db.exec("SELECT page.name, page.title FROM page, fts WHERE fts.text MATCH '" + keywords2 + "' AND page.id = fts.docid;");
  var html = '<h2>%s "' + query + '"</h2><br/><br/>';
  if (rs.length == 0) {
    results_div.innerHTML = html + "<br/>%s";
    return;
  }
  rs = rs[0]["values"];
  for(i in rs) {
    r = rs[i];
    html = html + "<div class='search_result'><b><a href='" + r[0] + "'>" + r[1] + "</a></b><div class='search_excerpt' id='search_excerpt" + i + "'>...</div></div>\n";
  }
  results_div.innerHTML = html;
  for(i in rs) {
    r = rs[i];
    excerpts(r[0], keywords, i);
  }
}

var remove_tag_regexp = new RegExp('<[^>]*>', "g");

function excerpts(name, keywords, excerpt_id) {
  var xmlhttp = new XMLHttpRequest();
  xmlhttp.open("GET", name, true);
  xmlhttp.responseType = "text";
  xmlhttp.onreadystatechange = function(){
    if(xmlhttp.status == 200 && xmlhttp.readyState == 4) {
      var page_html = xmlhttp.responseText;
      page_html = page_html.substring(page_html.search('<!-- DOCSTART -->') + 17);
      page_html = page_html.substring(0, page_html.search('<!-- DOCEND -->'));
      page_html = page_html.replace(remove_tag_regexp, "");
      page_html = " " + page_html + " ";
      
      var excerpts = "";
      for(i in keywords) {
        var keyword = keywords[i];
        var regexp = new RegExp(keyword, "gi");
        var p = page_html.search(regexp);
        var excerpt = page_html.substring(p - 50, p + 80);
        excerpt = excerpt.substring(excerpt.search(" "));
        excerpt = excerpt.substring(0, excerpt.lastIndexOf(" "));
        excerpt = excerpt.replace(regexp, "<span class='highlighted'>" + keyword + "</span>");
        if (!excerpt.endsWith(".")) { excerpt = excerpt + "..."; }
        excerpts = excerpts + "<div>" + excerpt + "</div>";
      }
      
      var excerpt_div = document.getElementById("search_excerpt" + excerpt_id);
      excerpt_div.innerHTML = excerpts;
    }
  };
  xmlhttp.send();
}
function on_search() {
  var input = document.getElementById("search_input");
  search(input.value);
}

var input = document.getElementById("search_input");
input.addEventListener("keyup", function(event) {
  event.preventDefault();
  if (event.keyCode === 13) on_search();
});
</script>
""" % (pages[-1].lang.langs[0],
       pages[-1].doc.blog.translate("search_result", pages[-1].lang),
       pages[-1].doc.blog.translate("search_no_result", pages[-1].lang))
    
    return html
    
  def htmlize(self, pages, page, lang):
    doc = page.doc
    lang.set_locale_current()

    javascripts = self.get_javascripts()
    if isinstance(page.doc, Search):
      javascripts.append("sql.js")
    styles  = "\n".join("""<link rel="stylesheet" href="__BLOG_HTML_ROOT__/_static/%s" type="text/css"/>""" % (stylesheet) for stylesheet in self.get_stylesheets())
    scripts = "\n".join("""<script type="text/javascript" src="__BLOG_HTML_ROOT__/_static/%s"></script>""" % (script) for script in javascripts)
    
    html = page.get_html()
    
    html = """<!-- DOCSTART -->
%s
<!-- DOCEND -->""" % html
    
    if   isinstance(doc, Post):
      html = """<div class="post-box section">
<div class="post-date container">%s</div>
%s
</div>""" % (page.doc.date.strftime("%x"), html)
    elif isinstance(doc, Archives):
      html = """<div class="section">
%s
%s
</div>""" % (html, self.archives(pages, lang))
    elif isinstance(doc, Search):
      html = """<div class="section">
%s
%s
</div>""" % (html, self.search(pages, lang))
    else:
      if html.strip():
        html = """<div class="section">
%s
</div>""" % html
      else:
        html = """<div>
%s
</div>""" % html
      
    if isinstance(doc, Category):
      nav_bar = self.nav_bar(pages, page)
      html += nav_bar
      
      posts = page.recent_posts
      if posts:
        html += """\n<div class="section" id="commentaires">"""
        for post in posts:
          post_html = shift_titles(post.get_html_pages()[lang].get_html_first_part())
          html += """<div class="post-box section">
<div class="post-date container">%s</div>
<div class="in-category container">%s %s</div>
%s
</div>
""" %  (post.date.strftime("%x"),
        doc.blog.ec_translations[(lang.langs[0], "in_category")],
        " ".join(category.get_html_page(lang).get_hyperlink() for category in post.categories),
        post_html)
        
        html += """\n</div>"""
        
      
    if len(pages) == 1:
      title = escape(pages[0].get_title())
    else:
      title = "%s — %s" % (escape(page.get_title()), escape(pages[0].get_title()))

    end = ""
    if isinstance(doc, Category):
      end = nav_bar
      
    html = """<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
%s
%s
<title>%s</title>
</head>
<body>
%s
%s
<div class="documentwrapper"><div class="bodywrapper"><div class="body" role="main">
%s
</div></div></div>
%s
<script>create_imageviewer();</script>
<div class="bottom">%s</div>
</body></html>""" % (styles, scripts,
                     title,
                     self.side_bar(pages, lang),
                     self.perspective_title(pages, page),
                     html,
                     end,
                     doc.blog.translate("bottom", page.lang),
)
    root = "../" * page.doc.doc_name.count("/")
    #if len(doc.blog.langs) == 1: current_lang = ""
    #else:                        current_lang = "_%s" % lang.langs[0]
    return html.replace("__BLOG_HTML_ROOT__/", root) #.replace("__BLOG_CURRENT_LANG__", current_lang)
  





# éClaircie
# Copyright (C) 2014-2018 Jean-Baptiste LAMY

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import eclaircie

def hash_link(s):
  return s.replace("<", "").replace(">", "").replace(" ", "").replace("@", "").replace(".", "").replace("/", "")

def email_link(email, name = "", label = "", localurl = None, url = None):
  if localurl == "auto":
    import eclaircie
    localurl = eclaircie.BLOG.blog_dir
  if url == "auto":
    import eclaircie
    url = eclaircie.BLOG.url
    
  if not label: label = email
  if name: email = "%s <%s>" % (name, email)
  div_id = "email_%s" % hash_link(email)
  email_link = """<a href="mailto:%s">%s</a>""" % (email, label)
  s  = """<div id="%s"></div><script>var x=document.getElementById('%s');""" % (div_id, div_id)
  if url:
    s += "if ((window.location.href.slice(0, %s)=='%s') | (window.location.href.slice(0, %s)=='%s')) { " % (len(localurl), localurl, len(url), url)
  s += "var y = '';"
  s += "".join("""y += '%s';""" % email_link[i*6:i*6+6] for i in range(len(email_link)//6+1))
  s += "x.innerHTML = y;"
  if url:
    s += "} else {x.innerHTML = '(hidden - not original page!)';}"
  s += """</script>"""
  return s.replace("@", "&#64") #.replace(".", "&#46")



# éClaircie
# Copyright (C) 2014-2018 Jean-Baptiste LAMY

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os, os.path, locale, urllib, urllib.request, re, codecs, glob
from datetime import date as Date
from html import escape as htmlescape
from docutils import nodes
from docutils.parsers.rst import Directive, directives


import eclaircie, eclaircie.rst, eclaircie.email_obfuscator

def register_directive(directive_class, directive_name = ""):
  directives.register_directive(directive_name or directive_class.__name__[:-9].lower(), directive_class)
  return directive_class

def register_node(node_class):
  setattr(eclaircie.rst.MyTranslator, "visit_%s" % node_class.__name__,  node_class.visit)
  setattr(eclaircie.rst.MyTranslator, "depart_%s" % node_class.__name__, node_class.depart)
  return node_class

  
@register_directive
class GalleryDirective(Directive):
  has_content = True
  
  def run(self):
    base_image_dir = os.path.join(eclaircie.BLOG.blog_dir, "_images")
    if not base_image_dir.endswith(os.path.sep): base_image_dir += os.path.sep
    
    images0 = (" ".join(self.content)).split()
    images = []
    for image0 in images0:
      if "*" in image0:
        filenames = sorted(glob.glob(os.path.join(eclaircie.BLOG.blog_dir, "_images", image0), recursive = True))
      else:
        filenames = [os.path.join(eclaircie.BLOG.blog_dir, "_images", image0)]
      for filename in filenames:
        if os.path.isdir(filename):
          images.extend(sorted(os.path.join(image0, f)
                               for f in os.listdir(filename)
                               if (not f.startswith("miniature_")) and (f.lower().endswith(".png") or f.lower().endswith(".jpeg") or f.lower().endswith(".jpg") or f.lower().endswith(".svg") or f.lower().endswith(".gif"))))
        else:
          images.append(filename[len(base_image_dir):])
    r = []
    next_id = 0
    for image in images:
      if   image.startswith("/"):
        pass
      elif image.startswith("http://") or image.startswith("."):
        r.append(GalleryImageNode(gallery_id = next_id, uri = image, mini = next_id, big = next_id)); next_id += 1
      else:
        dirname = os.path.dirname(image)
        if dirname: dirname = "%s/" % dirname
        big  = "__BLOG_HTML_ROOT__/_images/%s" % image
        mini = "__BLOG_HTML_ROOT__/_images/%sminiature_%s" % (dirname, os.path.basename(image))
        
        if image.endswith(".svg"):
          r.append(GalleryImageNode(gallery_id = next_id, uri = big,  mini = next_id, big = next_id)); next_id += 1
        else:
          r.append(GalleryImageNode(gallery_id = next_id, uri = mini, mini = next_id,     big = next_id + 1)); next_id += 1
          r.append(GalleryImageNode(gallery_id = next_id, uri = big,  mini = next_id - 1, big = next_id)); next_id += 1
    node = GalleryNode()
    node.extend(r)
    return [node]



@register_node
class GalleryNode(nodes.General, nodes.Element):
  @staticmethod
  def visit(self, node):
    self.body.append(self.starttag(node, 'div', CLASS = "gallery"))
    id_2_node = [child for child in node.children]
    
    big_images = []
    for image in id_2_node:
      if image.attributes["gallery_id"] == image.attributes["big"]:
        big = image.attributes["uri"]
        big_images.append("'%s'" % big)
    big_images = "[%s]" % ",".join(big_images)
    
    i = 0
    for image in id_2_node:
      if image.attributes["gallery_id"] == image.attributes["mini"]:
        mini = image.attributes["uri"]
        if mini.endswith(".svg"):
          extra = ' style="height:auto; width:auto; max-width:200px; max-height:150px;"'
        else:
          extra = ""
        self.body.append("""<img class="gallery-img" src="%s" onClick="show_imageviewer(%s, %s);"%s/>\n""" % (mini, big_images, i, extra))
        i += 1
        
  @staticmethod
  def depart(self, node):
    self.body.append('</div>')
    
@register_node
class GalleryImageNode(nodes.image):
  @staticmethod
  def visit (self, node): pass
  @staticmethod
  def depart(self, node): pass
  



@register_directive
class YoutubeDirective(Directive):
  has_content = True
  
  def run(self):
    node = YoutubeNode()
    node.attributes["youtube_id"] = " ".join(self.content).strip()
    return [node]

@register_node
class YoutubeNode(nodes.General, nodes.Element):
  @staticmethod
  def visit(self, node):
    youtube_id = node.attributes["youtube_id"]
    preview_filename = os.path.join(eclaircie.BLOG.blog_dir, "_images", "youtube_preview_%s.jpeg" % youtube_id)
    if not os.path.exists(preview_filename):
      s = urllib.request.urlopen("http://www.youtube.com/embed/%s" % youtube_id).read()
      s = s.decode("utf8")
      preview_url = re.findall('"iurlsd"\s*:\s*"(.*?)"', s)[0].replace("\\", "")
      s = urllib.request.urlopen(preview_url).read()
      open(preview_filename, "wb").write(s)
      
    self.body.append(self.starttag(node, 'div',
                                   style = """background-image: url("__BLOG_HTML_ROOT__/_images/youtube_preview_%s.jpeg");""" % youtube_id,
                                   onclick="""this.innerHTML="<iframe width='640' height='480' src='http://www.youtube.com/embed/%s?autoplay=1' frameborder='0' allowfullscreen='1'></iframe>";""" % youtube_id,
                                   CLASS = "video"))
    self.body.append(self.starttag(node, 'div', CLASS = "video-read"))
    self.body.append(' &gt; ')
    self.body.append('</div>')
    
  @staticmethod
  def depart(self, node):
    self.body.append('</div>')




@register_directive
class AudioDirective(Directive):
  has_content = True
  
  def run(self):
    node = AudioNode()
    node.attributes["filenames"] = [filename.strip() for filename in " ".join(self.content).split()]
    return [node]

EXT_2_MIME = { "ogg" : "audio/ogg", "mp3" : "audio/mpeg" }

@register_node
class AudioNode(nodes.General, nodes.Element):
  @staticmethod
  def visit(self, node):
    filenames = node.attributes["filenames"]
    
    self.body.append(self.starttag(node, 'audio', controls = "1"))
    for filename in filenames:
      self.body.append(self.starttag(node, 'source',
                                     src = "__BLOG_HTML_ROOT__/_downloads/%s" % filename,
                                     type = EXT_2_MIME[filename.rsplit(".")[-1].lower()]))
      self.body.append('</source>')
    self.body.append('Please update your browser for HTML5 audio support.')
    self.body.append('</audio>')
    
    for filename in filenames:
      self.body.append(self.starttag(node, 'p'))
      self.body.append(self.starttag(node, 'a', href = "__BLOG_HTML_ROOT__/_downloads/%s" % filename, CLASS = "reference download internal"))
      self.body.append(self.starttag(node, 'tt', CLASS = "xref download docutils literal"))
      self.body.append(self.starttag(node, 'span', CLASS = "pre"))
      self.body.append('%s</span></tt></a></p>' % filename)
  
  @staticmethod
  def depart(self, node): pass



@register_directive
class RedirectDirective(Directive):
  has_content = True
  
  def run(self):
    node = RedirectNode()
    node.attributes["redirect_url"] = " ".join(self.content).strip()
    return [node]

@register_node
class RedirectNode(nodes.General, nodes.Element):
  @staticmethod
  def visit(self, node):
    self.body.append(self.starttag(node, 'script'))
    self.body.append('window.location="%s";</script>' % node.attributes["redirect_url"])
    
  @staticmethod
  def depart(self, node): pass


@register_directive
class NabbleDirective(Directive):
  has_content = True
  
  def run(self):
    node = NabbleNode()
    contents = (" ".join(self.content)).split()
    node.attributes["nabble_name"] = contents[0]
    node.attributes["nabble_url"]  = contents[1]
    node.attributes["ids"].insert(0, "nabblelink")
    return [node]
  
@register_node
class NabbleNode(nodes.General, nodes.Element):
  @staticmethod
  def visit(self, node):
    self.body.append(self.starttag(node, "a", href = node.attributes["nabble_url"]))
    self.body.append(node.attributes["nabble_name"])
    self.body.append("</a>")
    self.body.append(self.starttag(node, "script", src = "%s/embed/f1" % node.attributes["nabble_url"]))
    self.body.append("</script>")
    
  @staticmethod
  def depart(self, node): pass



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
# éClaircie
# Copyright (C) 2014-2018 Jean-Baptiste LAMY

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os, os.path, sys, glob

HERE = os.path.dirname(sys.argv[0]) or "."

if len(sys.argv) <= 1: sys.argv.append("install")


#import distutils.sysconfig
#install_dir = distutils.sysconfig.get_python_lib()

import setuptools

setuptools.setup(
  name         = "eClaircie",
  version      = "0.3",
  license      = "GPLv3+",
  description  = "éClaircie is a 100% static and cloud-less blog engine.",
  long_description = open(os.path.join(HERE, "README.rst")).read(),
  
  author       = "Lamy Jean-Baptiste (Jiba)",
  author_email = "<jibalamy@free.fr>",
  url          = "https://bitbucket.org/jibalamy/eclaircie",
  classifiers  = [
    "Development Status :: 4 - Beta",
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    "Operating System :: POSIX :: Linux",
    "Environment :: Web Environment",
    "Topic :: Internet :: WWW/HTTP :: Site Management",
    ],
  
  requires = ["pillow"],
  zip_safe = False,
  
  scripts      = ["eclaircie"],
  package_dir  = {"eclaircie" : "."},
  packages     = ["eclaircie"],
  package_data = {
    "eclaircie" : ["themes/*/*/*.*", "themes/*/*.*", "themes/*.*"]
  }
  )
